@@ -1,5 +1,6 @@
 from enum import Enum, unique
 
+
 # This file is called ptypes because types is a reserved word in Python
 
 def auto(n_occurrences=1):
@@ -102,6 +103,17 @@ class TypeXML(AbstractType):
     ANNOTATIONS, DECISION, OUTPUT, VAR_HEURISTIC, VAL_HEURISTIC, PREPRO, SEARCH, RESTARTS, FILTERING = auto(9)
 
 
+UTF_EQ = "\u003D"
+UTF_NE = "\u2260"
+UTF_LT = "\uFE64"  # ""\u227A"
+UTF_LE = "\u2264"
+UTF_GE = "\u2265"
+UTF_GT = "\uFE65"  # "\u227B"
+UTF_LTGT = "\u2276"
+UTF_NOT_ELEMENT_OF = "\u00AC"  # ""\u2209"
+UTF_COMPLEMENT = "\u2201"
+
+
 @unique
 class TypeConditionOperator(AbstractType):
     LT, LE, GE, GT, EQ, NE, IN, NOTIN = auto(8)
@@ -126,6 +138,43 @@ class TypeConditionOperator(AbstractType):
                 return TypeConditionOperator.NE
         raise ValueError
 
+    @staticmethod
+    def to_utf(condition_operator):
+        if condition_operator == TypeConditionOperator.LT:
+            return UTF_LT
+        if condition_operator == TypeConditionOperator.LE:
+            return UTF_LE
+        if condition_operator == TypeConditionOperator.GE:
+            return UTF_GE
+        if condition_operator == TypeConditionOperator.GT:
+            return UTF_GT
+        if condition_operator == TypeConditionOperator.EQ:
+            return UTF_EQ
+        if condition_operator == TypeConditionOperator.NE:
+            return UTF_NE
+        if condition_operator == TypeConditionOperator.NOTIN:
+            return UTF_COMPLEMENT
+        assert False
+
+    def to_str(self):
+        if self is TypeConditionOperator.LT:
+            return "<"
+        if self is TypeConditionOperator.LE:
+            return "<="
+        if self is TypeConditionOperator.GE:
+            return ">="
+        if self is TypeConditionOperator.GT:
+            return ">"
+        if self is TypeConditionOperator.EQ:
+            return "="
+        if self is TypeConditionOperator.NE:
+            return "!="
+        if self is TypeConditionOperator.IN:
+            return "in"
+        if self is TypeConditionOperator.NOTIN:
+            return "not in"
+        assert False
+
 
 @unique
 class TypeOrderedOperator(Enum):
@@ -145,3 +194,16 @@ class TypeOrderedOperator(Enum):
 @unique
 class TypeRank(Enum):
     FIRST, LAST, ANY = auto(3)
+
+
+@unique
+class TypeSolver(Enum):
+    ACE, CHOCO = auto(2)
+
+
+@unique
+class TypeStatus(Enum):
+    UNSAT, SAT, OPTIMUM, CORE, UNKNOWN = auto(5)
+
+    def __str__(self):
+        return self.name
\ No newline at end of file
