@@ -8,7 +8,7 @@ from pycsp3.classes import main
 from pycsp3.classes.auxiliary.ptypes import auto
 from pycsp3.classes.main.variables import Variable
 from pycsp3.tools.inspector import checkType
-from pycsp3.tools.utilities import flatten, is_containing
+from pycsp3.tools.utilities import flatten, is_containing, warning
 from pycsp3.classes.auxiliary.ptypes import TypeCtr, TypeConditionOperator
 
 
@@ -129,6 +129,15 @@ class ECtr(Entity):
             self.constraint = c
             # CtrEntities.allEntities.append(self)
 
+    def __bool__(self):
+        warning(
+            "A constraint is evaluated as a Boolean (technically, __bool__ is called)"
+            + "\n\tIt is likely a problem with the use of logical operators"
+            + "\n\tFor example, you must write Or(AllDifferent(x), (x[0] == x[2])) instead of AllDifferent(x) or (x[0] == x[2])"
+            + "\n\t or you must post separately the two constraints"
+            + "\n\tSee also the end of section about constraint Intension in chapter 'Twenty popular constraints' of the guide\n")
+        return True
+
 
 class ECtrs(Entity):
     """ Class for representing sets of constraints """
@@ -138,6 +147,22 @@ class ECtrs(Entity):
         assert isinstance(constraints, list)
         self.entities = [c for c in constraints if c is not None]
 
+    def _flat_constraints(self, t):
+        for e in self.entities:
+            if isinstance(e, ECtr):
+                t.append(e.constraint)
+            elif isinstance(e, ECtrs):
+                e._flat_constraints(t)
+            elif isinstance(e, EMetaCtr):
+                t.append(e)
+        return t
+
+    def flat_constraints(self):
+        return self._flat_constraints([])
+
+    def __repr__(self):
+        return "\n".join(str(e) for e in self.flat_constraints())
+
 
 class EToGather(ECtrs):
     ''' Constraints possibly stored in a group (the user asked to gather these constraints)'''
@@ -156,8 +181,13 @@ class EToSatisfy(ECtrs):
             CtrEntities.items.append(self)
             super().__init__(constraints)
 
-    def delete(self, i):
-        del self.entities[i]
+    def delete(self, i=None):
+        if i is None:
+            self.entities = []
+        elif len(self.entities) == 1 and isinstance(self.entities[0], ECtrs):
+            del self.entities[0].entities[i]
+        else:
+            del self.entities[i]
 
 
 class EGroup(ECtrs):
@@ -193,6 +223,9 @@ class EMetaCtr(Entity):
         assert len(self.entities) >= min_arity, "At least " + str(min_arity) + " components must be specified in the meta-constraint"
         assert max_arity is None or len(self.entities) <= max_arity, "At most " + str(max_arity) + " components must be specified in the meta-constraint"
 
+    def __repr__(self):
+        return str(self.id) + "(" + ",".join(str(e.constraint) for e in self.entities) + ")"
+
 
 class EAnd(EMetaCtr):
     def __init__(self, constraints):
@@ -278,7 +311,7 @@ class VarEntities:
 
 
 class CtrEntities:
-    items = []
+    items = []  # contains EToSatisfy objects
 
 
 class ObjEntities:
@@ -290,6 +323,23 @@ class AnnEntities:
     items_types = []
 
 
+def clear():
+    """
+    Removes everything that was declared (variables) or posted (constraints, objective)
+    """
+    VarEntities.items = []
+    VarEntities.varToEVar = dict()
+    VarEntities.varToEVarArray = dict()
+    VarEntities.prefixToEVarArray = dict()
+    CtrEntities.items = []
+    ObjEntities.items = []
+    AnnEntities.items = []
+    AnnEntities.items_types = []
+    Variable.name2obj = dict()
+    main.constraints.auxiliary.obj = None
+    # Diffs.reset()
+
+
 @unique
 class TypeNode(Enum):
     def __init__(self, id, min_arity, max_arity):
@@ -302,7 +352,7 @@ class TypeNode(Enum):
         return self.lowercase_name
 
     ''' 0-ary '''
-    VAR, PAR, INT, RATIONAL, DECIMAL, SYMBOL, PARTIAL = ((id, 0, 0) for id in auto(7))
+    VAR, INT, RATIONAL, DECIMAL, SYMBOL, PARTIAL, COL = ((id, 0, 0) for id in auto(7))
 
     ''' Unary'''
     NEG, ABS, SQR, NOT, CARD, HULL, CONVEX, SQRT, EXP, LN, SIN, COS, TAN, ASIN, ACOS, ATAN, SINH, COSH, TANH = ((id, 1, 1) for id in auto(19))
@@ -340,17 +390,17 @@ class TypeNode(Enum):
         if isinstance(v, TypeNode):
             return v
         if isinstance(v, str):
-            if v == "<":
+            if v in ("<", "lt"):
                 return TypeNode.LT
-            if v == "<=":
+            if v in ("<=", "le"):
                 return TypeNode.LE
-            if v == ">=":
+            if v in (">=", "ge"):
                 return TypeNode.GE
-            if v == ">":
+            if v in (">", "gt"):
                 return TypeNode.GT
-            if v in ("=", "=="):
+            if v in ("=", "==", "eq"):
                 return TypeNode.EQ
-            if v in ("!=", "<>"):
+            if v in ("!=", "<>", "ne"):
                 return TypeNode.NE
             return TypeNode[v.upper()]
         if isinstance(v, TypeConditionOperator):
@@ -392,6 +442,14 @@ class Node(Entity):
         self.abstractTree = None
         self.abstractValues = None
 
+    def __bool__(self):
+        warning(
+            "A node is evaluated as a Boolean (technically, __bool__ is called)"
+            + "\n\tIt is likely a problem with the use of logical operators"
+            + "\n\tFor example, you must write (x[0] == x[1])  | (x[0] == x[2]) instead of (x[0] == x[1])  or (x[0] == x[2])"
+            + "\n\tSee also the end of section about constraint Intension in chapter 'Twenty popular constraints' of the guide\n")
+        return True
+
     def eq__safe(self, other):
         if not isinstance(other, Node) or self.type != other.type or self.leaf != other.leaf:
             return False
@@ -399,6 +457,19 @@ class Node(Entity):
             return len(self.sons) == len(other.sons) and all(self.sons[i].eq__safe(other.sons[i]) for i in range(len(self.sons)))
         return self.sons.eq__safe(other.sons) if isinstance(self.sons, Variable) else self.sons == other.sons
 
+    def __strsmart__(self):
+        if self.type.is_leaf():
+            if self.type == TypeNode.COL:
+                return "%" + str(self.sons)
+            return str(self.sons)
+        if self.type == TypeNode.ADD or self.type == TypeNode.SUB:
+            msg = "Smart tuple must be of the form {eq|lt|le|ge|gt|ne}{var|integer}{+|-}{var|integer}"
+            assert len(self.sons) == 2, msg
+            assert self.sons[0].type in (TypeNode.INT, TypeNode.COL) and self.sons[1].type in (TypeNode.INT, TypeNode.COL), msg
+            return self.sons[0].__strsmart__() + ("+" if self.type == TypeNode.ADD else "-") + self.sons[1].__strsmart__()
+        else:
+            assert False, "Smart tuple must be of the form col(x)[+or-][integer]"
+
     def __str__(self):
         return str(self.sons) if self.type.is_leaf() else str(self.type) + "(" + ",".join(str(son) for son in self.sons) + ")"
 
