@@ -10,7 +10,7 @@ from pycsp3.classes.main.domains import Domain
 from pycsp3.classes.main.variables import Variable, VariableInteger
 from pycsp3.dashboard import options
 from pycsp3.tools import curser
-from pycsp3.tools.utilities import ANY, is_1d_list, matrix_to_string, integers_to_string, table_to_string, flatten, is_matrix, error, \
+from pycsp3.tools.utilities import ANY, is_1d_list, matrix_to_string, integers_to_string, table_to_string, flatten, is_matrix, error, error_if, \
     to_ordinary_table, warning, is_windows
 
 
@@ -66,7 +66,8 @@ class ConstraintArgument:
         # return self.content == other.content
 
     def __str__(self):
-        return str(self.name) + ":" + str(self.content)
+        content = "[" + ", ".join(str(v) for v in self.content) + "]" if isinstance(self.content, list) else self.content
+        return str(self.name) + ":" + str(content)
 
 
 class Constraint:
@@ -141,7 +142,16 @@ class Constraint:
         return " ".join("%" + str(v + self.n_parameters - length) for v in range(length))
 
     def __str__(self):
-        return str(self.name) + ": " + " ".join(str(a) for a in self.attributes) + " ".join(str(v) for k, v in self.arguments.items())
+        items = list(self.arguments.items())
+        condition = None  # items[-1][1].content.infix_string() if items[-1][0] == TypeCtrArg.CONDITION else None
+        s_attributes = " ".join(str(t.name) + ": " + str(v) for (t, v) in self.attributes if t is not TypeCtrArg.TYPE)
+        s_arguments = ", ".join(str(v) for k, v in (items if condition is None else items[:-1]) if v.content is not None)
+        body = ("[" + s_attributes + "]" if len(s_attributes) > 0 else "") + "(" + s_arguments + ")"
+        if len(self.attributes) > 0 and self.attributes[0][0] is TypeCtrArg.TYPE:  # objective
+            s = str(self.name) + "(" + str(self.attributes[0][1]) + body + ")"
+        else:
+            s = str(self.name) + body
+        return s if condition is None else s + " " + condition
 
 
 class ConstraintUnmergeable(Constraint):
@@ -212,7 +222,7 @@ class ConstraintExtension(Constraint):
             if tpl:
                 table[i] = tuple(tpl)
 
-        return sorted(list(to_ordinary_table(table, [x.dom for x in scope], keep_any=True)))
+        return sorted(list(to_ordinary_table(table, [x.dom for x in scope], starred=True)))
 
     @staticmethod
     def remove_redundant_tuples(table):
@@ -232,7 +242,6 @@ class ConstraintExtension(Constraint):
     def process_table(self, scope, table):
         if len(table) == 0:
             return None
-
         # we compute the hash code of the table
         try:
             h = hash(tuple(table))
@@ -276,11 +285,13 @@ class ConstraintExtension(Constraint):
                 # if ever we would want to use caching, we should include domains when computing hashes
                 table = ConstraintExtension.convert_smart_to_ordinary(scope, table)
                 table = ConstraintExtension.remove_redundant_tuples(table)
+
                 return table_to_string(table, restricting_domains=[x.dom for x in scope] if self.restrictTablesWrtDomains else None,
                                        parallel=possible_parallelism)
 
     def __init__(self, scope, table, positive=True, keepsmartconditions=False, restrictTablesWrtDomains=False):
         super().__init__(TypeCtr.EXTENSION)
+
         self.keepsmartconditions = keepsmartconditions
         self.restrictTablesWrtDomains = restrictTablesWrtDomains
         assert is_1d_list(scope, Variable)
@@ -345,9 +356,10 @@ class ConstraintAllDifferentList(ConstraintUnmergeable):
 
 
 class ConstraintAllDifferentMatrix(ConstraintUnmergeable):
-    def __init__(self, lst):
+    def __init__(self, lst, excepting):
         super().__init__(TypeCtr.ALL_DIFFERENT)
         self.arg(TypeCtrArg.MATRIX, matrix_to_string(lst), content_compressible=lst)
+        self.arg(TypeCtrArg.EXCEPT, excepting)
 
 
 class ConstraintAllEqual(Constraint):
@@ -744,6 +756,8 @@ class PartialConstraint:  # constraint whose condition has not been given such a
         if isinstance(obj1, ScalarProduct):
             obj1 = PartialConstraint(ConstraintSum(obj1.variables, obj1.coeffs, None))  # to be sure to have at least one PartialConstraint
         assert isinstance(obj1, PartialConstraint) or isinstance(obj2, PartialConstraint)
+        if obj2 is None:
+            return obj1
         inverted, obj1, obj2 = (False, obj1, obj2) if isinstance(obj1, PartialConstraint) else (True, obj2, obj1)
         pair = obj2.var_val_if_binary_type(TypeNode.MUL) if isinstance(obj2, Node) else None
         if pair:
@@ -769,7 +783,7 @@ class ScalarProduct:
     def __init__(self, variables, coefficients):
         variables = list(variables) if isinstance(variables, tuple) else variables
         coefficients = list(coefficients) if isinstance(coefficients, tuple) else coefficients
-        assert isinstance(variables, list) and isinstance(coefficients, (int, list, range)), variables
+        assert isinstance(variables, list) and isinstance(coefficients, (int, list, range)), str(variables) + " " + str(coefficients)
         self.variables = flatten(variables)  # for example, in order to remove None occurrences
         self.coeffs = flatten([coefficients] * len(variables) if isinstance(coefficients, int) else coefficients)
         assert len(self.variables) == len(self.coeffs), str(self.variables) + " " + str(self.coeffs)
@@ -821,7 +835,12 @@ class _Auxiliary:
         self.prefix = "aux_gb"
         self.cache = []
 
-    def __replace(self, obj, dom):
+    def __clear(self):
+        self._introduced_variables = []
+        self._collected_constraints = []
+        self.cache = []
+
+    def __replace(self, obj, dom, *, systematically_append_obj=True):
         assert dom.get_type() == TypeVar.INTEGER
         index = len(self._introduced_variables)
         name = self.prefix + "[" + str(index) + "]"
@@ -831,7 +850,7 @@ class _Auxiliary:
             self._introduced_variables = EVarArray([var], self.prefix, self.prefix + "[i] is the ith auxiliary variable having been automatically introduced")
         else:
             self._introduced_variables.extend_with(var)
-        if obj:
+        if systematically_append_obj or obj:
             self._collected_constraints.append((obj, var))
         return var
 
@@ -868,7 +887,7 @@ class _Auxiliary:
         if all(0 <= v < length for v in index.dom):
             return None
         values = possible_range({v for v in index.dom if 0 <= v < length})
-        return self.__replace(None, Domain(values))
+        return self.__replace(None, Domain(values), systematically_append_obj=False)
 
     def collected(self):
         t = self._collected_constraints
@@ -877,7 +896,7 @@ class _Auxiliary:
 
 
 def auxiliary():
-    if not hasattr(auxiliary, "obj"):
+    if not hasattr(auxiliary, "obj") or auxiliary.obj is None:
         auxiliary.obj = _Auxiliary()
     return auxiliary.obj
 
@@ -894,8 +913,9 @@ def global_indirection(c):
         index = c.arguments[TypeCtrArg.INDEX].content
         length = len(c.arguments[TypeCtrArg.LIST].content)
         aux = auxiliary().replace_element_index(length, index)
-        if aux:
+        if aux:  # this is the case if we need another variable to have a correct indexing
             c.arguments[TypeCtrArg.INDEX].content = aux
+            # below, should we replace ANY by a specific value (for avoid interchangeable values)?
             functions.satisfy((index, aux) in {(v, v if 0 <= v < length else ANY) for v in index.dom})
     if isinstance(c, ConstraintAllDifferent):
         lst = c.arguments[TypeCtrArg.LIST].content
@@ -917,9 +937,23 @@ def manage_global_indirection(*args):
     assert len(args) > 1
     if any(isinstance(arg, EMetaCtr) for arg in args):
         return None
+
+    msg = "A subexpression node is evaluated as a Boolean (False or True)" + \
+          "\n\tIt is likely a problem with the use of logical operators" + \
+          "\n\tFor example, you must write (x[0] == x[1])  | (x[0] == x[2]) instead of (x[0] == x[1])  or (x[0] == x[2])" + \
+          "\n\tSee also the end of section about constraint Intension in chapter 'Twenty popular constraints' of the guide\n"
+
     t = []
     for arg in args:
-        if isinstance(arg, ECtr):
+        if arg is True:  # means that we must have a unary subexpression of the form 'x in S' in a more general expression
+            error_if(len(curser.queue_in) == 0, msg)
+            (values, x) = curser.queue_in.pop()
+            arg = functions.belong(x, values)
+        elif arg is False:  # means that we must have a unary subexpression of the form 'x not in S' in a more general expression
+            error_if(len(curser.queue_in) == 0, msg)
+            (values, x) = curser.queue_in.pop()
+            arg = functions.not_belong(x, values)
+        elif isinstance(arg, ECtr):
             gi = global_indirection(arg.constraint)
             if gi is None:
                 return None
