@@ -7,6 +7,8 @@ from pycsp3.classes.main.domains import Domain
 class Variable:
     name2obj = dict()  # Dictionary (keys: names of variables - values: variable objects)
 
+    arrays = []  # the arrays of variables, as introduced by the user in the model
+
     @staticmethod
     def build_names_array(name, sizes, mins, indexes=[]):
         if sizes:
@@ -78,12 +80,14 @@ class Variable:
             self.indexes = [int(v) for v in re.split("\]\[", self.suffix[1:-1])]
         self.inverse = inverse  # arithmetic inverse
         self.negation = negation  # logical negation
+        self.value = None  # value of the last found solution
+        self.values = []  # values of the successive found solutions
 
     def eq__safe(self, other):
         return isinstance(other, Variable) and self.id == other.id
 
     def __eq__(self, other):
-        return self.eq__safe(other)  #isinstance(other, Variable) and self.id == other.id
+        return self.eq__safe(other)  # isinstance(other, Variable) and self.id == other.id
 
     def __invert__(self):
         return Variable(self.id, self.dom, negation=not self.negation)
