@@ -45,7 +45,7 @@ if not variant():
 
 elif variant("compact"):
     # coins[i] is the number of coins of the ith type
-    c1, c5, c10, c20, c50, e1, e2 = coins = VarArray(size=7, dom=range(50))
+    coins = VarArray(size=7, dom=range(50))
 
     satisfy(
         # the given change must be correct
