@@ -1,20 +1,20 @@
-import subprocess
-import time
-import sys
 import os
 import signal
+import subprocess
+import sys
 import uuid
-from io import IOBase
+import re
 
 from lxml import etree
-# from py4j.java_gateway import JavaGateway, Py4JNetworkError
 
-from pycsp3.classes.entities import VarEntities, EVar
-from pycsp3.classes.main.variables import Variable
+from pycsp3.classes.auxiliary.ptypes import TypeStatus
+from pycsp3.classes.entities import VarEntities, EVar, EVarArray
+from pycsp3.classes.main.variables import Variable, VariableInteger
 from pycsp3.dashboard import options
-from pycsp3.tools.utilities import Stopwatch, flatten, GREEN, WHITE, is_windows
+from pycsp3.tools.utilities import Stopwatch, flatten, GREEN, WHITE, is_windows, ANY
 
-UNKNOWN, SAT, UNSAT, OPTIMUM = "UNKNOWN", "SAT", "UNSAT", "OPTIMUM"
+
+# from py4j.java_gateway import JavaGateway, Py4JNetworkError
 
 
 def process_options(solving):
@@ -105,12 +105,13 @@ def process_options(solving):
 
 
 class Logger:
-    def __init__(self, extend_filename=""):
+    def __init__(self, extend_filename="", verbose=0):
         mac, pid = hex(uuid.getnode()), str(os.getpid())
         filename = "solver_" + mac + "_" + pid + "_" + (str(extend_filename) if extend_filename else "") + ".log"
         self.log_file = os.getcwd() + os.sep + filename
         # self.log_file = os.path.dirname(os.path.realpath(__file__)) + os.sep + filename  # old code
-        print("  * Log file of the solver:", self.log_file)
+        if verbose > 0:
+            print("    - logfile:", self.log_file)
         if os.path.exists(self.log_file):
             os.remove(self.log_file)
         self.log = open(self.log_file, "a")
@@ -131,10 +132,11 @@ class Logger:
 
 
 class Instantiation:
-    def __init__(self, pretty_solution, variables, values):
-        self.pretty_solution = pretty_solution
+    def __init__(self, root, variables, values, pretty_solution):
+        self.root = root
         self.variables = variables
         self.values = values
+        self.pretty_solution = pretty_solution
 
     def __repr__(self):
         return self.variables, self.values
@@ -154,50 +156,50 @@ class SolverProcess:
         self.stdout = None
         self.stderr = None
         self.last_command_wck = None
-        self.extend_filename_logger = None
+        self.log_filename_suffix = None
         self.n_executions = 0
+        self.last_log = None
+        # concerning the last execution:
+        self.last_solution = None
+        self.n_solutions = None
+        self.bound = None
+        self.status = None
+        # concerning extraction:
+        self.core = None
 
     def command(self, _command):
         self.command = _command
 
-    def setting(self, option=""):
-        option = str(option).strip()
-        self.options = " " + option if self.options != "" else option
+    def get_logger(self):
+        return self.last_log
 
-    def extend_logger(self, _extend_filename_logger):
-        self.extend_filename_logger = _extend_filename_logger
+    def setting(self, value):
+        if value is not None:
+            value = str(value).strip()
+            self.options = " " + value if self.options != "" else value
+
+    def log_suffix(self, _extend_filename_logger):
+        self.log_filename_suffix = _extend_filename_logger
 
     def parse_general_options(self, string_options, dict_options, dict_simplified_options):  # specific options via args are managed automatically
         raise NotImplementedError("Must be overridden")
 
-    def solve(self, instance, string_options="", dict_options=dict(), dict_simplified_options=dict(), compiler=False, *, verbose=False, automatic=False):
+    def _solve(self, instance, string_options="", dict_options=dict(), dict_simplified_options=dict(), compiler=False, *, verbose=0, automatic=False,
+               extraction=False):
         model, cop = instance
+        if extraction:
+            self.switch_to_extraction()
 
-        def extract_result_and_solution(stdout):
-            if stdout.find("<unsatisfiable") != -1 or stdout.find("s UNSATISFIABLE") != -1:
-                return UNSAT, None
-            if stdout.find("<instantiation") == -1 or stdout.find("</instantiation>") == -1:
-                print("  Actually, the instance was not solved")
-                return UNKNOWN, None
-            left, right = stdout.rfind("<instantiation"), stdout.rfind("</instantiation>")
-            s = stdout[left:right + len("</instantiation>")].replace("\nv", "")
-            root = etree.fromstring(s, etree.XMLParser(remove_blank_text=True))
-            optimal = stdout.find("s OPTIMUM ") != -1
-            if cop:
-                if "type" not in root.attrib:
-                    root.attrib['type'] = "solution" + (" optimal" if optimal else "")
-                elif root.attrib['type'] == "solution" and optimal:
-                    root.attrib['type'] = "solution optimal"
-                if "cost" not in root.attrib:
-                    left = stdout.rfind("o ") + 2
-                    right = left + 1
-                    while stdout[right].isdigit():
-                        right += 1
-                    root.attrib['cost'] = stdout[left:right]
-                if "id" in root.attrib:
-                    del root.attrib['id']
+        def _int_from(s, left):
+            right = left + s[left:].find("\n")
+            left = right - 1
+            while s[left].isdigit():
+                left -= 1
+            return int(s[left + 1:right])
+
+        def _record_solution(roots, i):
             variables = []
-            for token in root[0].text.split():
+            for token in roots[i][0].text.split():
                 r = VarEntities.get_item_with_name(token)
                 if isinstance(r, EVar):
                     variables.append(r.variable)
@@ -206,8 +208,12 @@ class SolverProcess:
                 else:
                     for x in flatten(r.variables, keep_none=True):
                         variables.append(x)
+            if i == 0:  # reset the history in that case
+                for x in variables:
+                    if x:
+                        x.values = []
             values = []
-            for tok in root[1].text.split():
+            for tok in roots[i][1].text.split():
                 if 'x' in tok:  # in order to handle compact forms in solutions
                     vk = tok.split('x')
                     assert len(vk) == 2
@@ -217,13 +223,71 @@ class SolverProcess:
                     values.append(tok)
             # values is a list with all values given as strings (possibly '*')
             assert len(variables) == len(values)
-            for i, v in enumerate(values):
+            for i, _ in enumerate(values):
                 if variables[i]:
-                    variables[i].value = v  # we add a new field (may be useful)
-            pretty_solution = etree.tostring(root, pretty_print=True, xml_declaration=False).decode("UTF-8").strip()
-            return OPTIMUM if optimal else SAT, Instantiation(pretty_solution, variables, values)
+                    if isinstance(variables[i], VariableInteger):
+                        values[i] = int(values[i]) if values[i] != "*" else ANY
+                    variables[i].value = values[i]  # we record the last found solution value
+                    variables[i].values.append(values[i])  # we record it in the history
+            return variables, values
 
-        def execute(command, verbose):
+        def extract_result_and_solution(stdout):
+            if extraction:
+                left = stdout.rfind("c CORE")
+                if left == -1:
+                    return TypeStatus.UNKNOWN
+                right = left + stdout[left:].find("\n")
+                self.core = stdout[left:right]
+                return TypeStatus.CORE
+
+            if stdout.find("<unsatisfiable") != -1 or stdout.find("s UNSATISFIABLE") != -1:
+                return TypeStatus.UNSAT
+            if stdout.find("<instantiation") == -1 or stdout.find("</instantiation>") == -1:
+                print("  Actually, the instance was not solved")
+                return TypeStatus.UNKNOWN
+
+            if "limit=no" in string_options or ("limit_sols" in dict_simplified_options and int(dict_simplified_options["limit_sols"]) > 1):
+                # TODo findall does not seem to work with the output of Choco. why?
+                roots = [etree.fromstring(("<instantiation" + tok + "</instantiation>").replace("\nv", ""), etree.XMLParser(remove_blank_text=True))
+                         for tok in re.findall(r"<instantiation(.*?)</instantiation>", stdout)]
+            else:
+                left, right = stdout.rfind("<instantiation"), stdout.rfind("</instantiation>")
+                roots = [etree.fromstring(stdout[left:right + len("</instantiation>")].replace("\nv", ""), etree.XMLParser(remove_blank_text=True))]
+
+            for i in range(len(roots) - 1):  # all roots except the last one to record the history
+                _record_solution(roots, i)
+
+            root = roots[-1]
+            variables, values = _record_solution(roots, len(roots) - 1)
+            optimal = stdout.find("s OPTIMUM") != -1
+            if cop:
+                root.attrib['type'] = "optimum" if optimal else "solution"
+                if "cost" not in root.attrib:
+                    root.attrib['cost'] = _int_from(stdout, stdout.rfind("o ") + 2)
+                self.bound = int(root.attrib['cost'])
+                if "id" in root.attrib:
+                    del root.attrib['id']
+
+            def _array_values(t):
+                if t is None:
+                    return None
+                if isinstance(t, Variable):
+                    return t.value
+                t.values = [_array_values(v) for v in t]
+                return t.values
+
+            for array in Variable.arrays:
+                _array_values(array)  # we set the value of the field 'values'
+                # (currently, the recursive mode does not work; superficial copy when getting an item)
+
+            pretty_solution = etree.tostring(root, pretty_print=True, xml_declaration=False).decode("UTF-8").strip()
+            self.last_solution = Instantiation(root, variables, values, pretty_solution)
+            j = stdout.find("d FOUND SOLUTIONS")
+            if j != -1:
+                self.n_solutions = _int_from(stdout, j)
+            return TypeStatus.OPTIMUM if optimal else TypeStatus.SAT
+
+        def execute(command):
             if not is_windows():
                 p = subprocess.Popen(command.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True, preexec_fn=os.setsid)
             else:
@@ -238,9 +302,10 @@ class SolverProcess:
 
             signal.signal(signal.SIGINT, new_handler)
             log = Logger(
-                self.extend_filename_logger if self.extend_filename_logger is not None else str(self.n_executions))  # To record the output of the solver
+                self.log_filename_suffix if self.log_filename_suffix is not None else str(self.n_executions), verbose)  # To record the output of the solver
+            self.last_log = log.log_file
             for line in p.stdout:
-                if verbose:
+                if verbose == 2:
                     sys.stdout.write(line)
                 log.write(line)
             p.wait()
@@ -249,7 +314,7 @@ class SolverProcess:
             signal.signal(signal.SIGINT, handler)  # Reset the right SIGINT
             return log.read(), stopped
 
-        if len(VarEntities.items) == 0:
+        if model is not None and len(VarEntities.items) == 0:
             print("\n The instance has no variable, so the solver is not run.")
             print("Did you forget to indicate the variant of the model?")
             return None
@@ -268,25 +333,40 @@ class SolverProcess:
         stopwatch = Stopwatch()
         solver_args = self.parse_general_options(string_options, dict_options, dict_simplified_options)
         solver_args += " " + dict_options["args"] if "args" in dict_options else ""
-        solver_args += self.options
+        solver_args += " " + self.options
+
+        verbose = 2 if options.solve or "verbose" in dict_simplified_options else verbose
+        command = self.command + " " + (model if model is not None else "") + " " + solver_args
+
+        if verbose > 0:
+            print("\n  * Solving by " + self.name + " in progress ... ")
+            print("    - command:", command)
+        out_err, stopped = execute(command)
 
-        verbose = verbose or options.solve or "verbose" in dict_simplified_options
-        command = self.command + " " + model + " " + solver_args
-        print("\n  * Solving by " + self.name + " in progress ... ")
-        print("    with command: ", command)
-        out_err, stopped = execute(command, verbose)
-        print()
         missing = out_err is not None and out_err.find("Missing Implementation") != -1
         self.last_command_wck = stopwatch.elapsed_time()
-        if stopped:
-            print("  * Solving process stopped (SIGINT) by " + self.name + " after " + GREEN + self.last_command_wck + WHITE + " seconds")
-        else:
-            print("  * Solved by " + self.name + " in " + GREEN + self.last_command_wck + WHITE + " seconds")
-        if missing:
-            print("\n   This is due to a missing implementation")
-        print("\n  NB: use the solver option v, as in -solver=[choco,v] or -solver=[ace,v] to see directly the output of the solver.\n")
+        if verbose > 0:
+            if stopped:
+                print("  * Solving process stopped (SIGINT) by " + self.name + " after " + GREEN + self.last_command_wck + WHITE + " seconds")
+            else:
+                print("\n  * Solved by " + self.name + " in " + GREEN + self.last_command_wck + WHITE + " seconds")
+            if missing:
+                print("\n   This is due to a missing implementation")
+            if automatic and verbose < 2:
+                print("\n  NB: use the solver option v, as in -solver=[choco,v] or -solver=[ace,v] to see directly the output of the solver.")
+            else:
+                print()
         self.n_executions += 1
-        return extract_result_and_solution(out_err) if out_err else (None, None)
+        return extract_result_and_solution(out_err) if out_err else TypeStatus.UNKNOWN
+
+    def solve(self, instance, string_options="", dict_options=dict(), dict_simplified_options=dict(), compiler=False, *, verbose=0, automatic=False,
+              extraction=False):
+        self.status = self._solve(instance, string_options, dict_options, dict_simplified_options, compiler, verbose=verbose, automatic=automatic,
+                                  extraction=extraction)
+        return self.status
+
+    def switch_to_extraction(self):
+        pass
 
 # class SolverPy4J(SolverProcess):  # TODO in progress
 #     gateways = []
