@@ -5,6 +5,7 @@ from pycsp3.classes.auxiliary.ptypes import TypeCtrArg
 from pycsp3.classes.auxiliary.conditions import Condition
 from pycsp3.classes.entities import CtrEntities, ECtr, ECtrs, EMetaCtr, EGroup, EBlock, EToGather, ESlide, EToSatisfy, TypeNode
 from pycsp3.classes.main.constraints import ConstraintIntension, Diffs, ConstraintInstantiation
+from pycsp3.classes.main.variables import Variable
 from pycsp3.tools.utilities import error
 
 LIMIT_FOR_VAR_ARGS = 3
@@ -18,6 +19,7 @@ def build_similar_constraints():
 
 
 # Phase 1: Detecting groups of similar constraints
+
 def detecting_groups_recursively(ctr_entities):
     def _detecting_groups(entities):
         flags = [False] * len(entities)  # indicate (indexes of) constraints that are similar at a given moment (see intern loop)
@@ -54,21 +56,27 @@ def detecting_groups_recursively(ctr_entities):
 
 # Phase 2: Building groups (constraint templates and args)
 
-
 def _compute_group_abstraction_intension(group):
     def _replace_parameter(s, k, v):
         return (s.replace("(" + k + ")", "(" + v + ")").replace("," + k + ",", "," + v + ",").replace("(" + k + ",", "(" + v + ",")
                 .replace("," + k + ")", "," + v + ")").replace("(" + k + ")", "(" + v + ")"))
 
+    def _same(v1, v2):
+        if isinstance(v1, Variable):
+            return v1.eq__safe(v2)
+        if isinstance(v2, Variable):
+            return v2.eq__safe(v1)
+        return v1 == v2
+
     def _is_same_value_at_column(i, all_args):
         """ comparison from both ends so as to find opportunistically that the parameter is not the same everywhere"""
         left, right = 1, len(all_args) - 1
         value = all_args[0][i]
         while True:
-            if value != all_args[left][i]:
+            if not _same(value, all_args[left][i]):
                 return False
             left += 1
-            if value != all_args[right][i]:
+            if not _same(value, all_args[right][i]):
                 return False
             right -= 1
             if left > right:
@@ -78,10 +86,10 @@ def _compute_group_abstraction_intension(group):
     def _is_same_value_in_columns(i, j, all_args):
         left, right = 0, len(all_args) - 1
         while True:
-            if all_args[left][i] != all_args[left][j]:
+            if not _same(all_args[left][i], all_args[left][j]):
                 return False
             left += 1
-            if all_args[right][i] != all_args[right][j]:
+            if not _same(all_args[right][i], all_args[right][j]):
                 return False
             right -= 1
             if left > right:
@@ -204,6 +212,7 @@ def building_groups_recursively(entities, previous=None):
 
 
 # Phase 3: adding/removing some blocks
+
 def canonizing_groups_and_blocks(entities, previous=None):
     def _building_block(entity):
         if len(entity.entities) == 0:
@@ -241,6 +250,7 @@ def canonizing_groups_and_blocks(entities, previous=None):
 
 
 # Phase 4: detecting forms corresponding to instantiations
+
 def recognizing_instantiations(entities):
     def _elements_for_building_instantiation(trees):
         t1, t2 = [], []
