@@ -1,6 +1,7 @@
 import types
 from collections import deque, namedtuple
 
+from pycsp3 import functions
 from pycsp3.classes.entities import Node, TypeNode, ECtr, EMetaCtr
 from pycsp3.classes.main.constraints import (
     ScalarProduct, PartialConstraint, ConstraintSum, ConstraintElement, ConstraintElementMatrix,
@@ -9,9 +10,8 @@ from pycsp3.classes.main.constraints import (
 from pycsp3.classes.main.variables import Variable, VariableInteger
 from pycsp3.libs.forbiddenfruit import curse
 from pycsp3.tools.inspector import checkType
-from pycsp3.tools.utilities import flatten, is_containing, unique_type_in, is_1d_tuple, is_1d_list, is_matrix, ANY, warning, error_if, error
-
-from pycsp3 import functions
+from pycsp3.tools.utilities import flatten, is_containing, unique_type_in, is_1d_tuple, is_1d_list, is_matrix, is_square_matrix, ANY, structured_list, warning, \
+    error_if
 
 queue_in = deque()  # To store partial constraints when using the IN operator
 
@@ -76,6 +76,13 @@ def cursing():
         if isinstance(other, int) and (is_1d_list(self, Variable) or is_1d_tuple(self, Variable)):  # member/element constraint
             queue_in.append((self, other))
             return True
+        if isinstance(other, (tuple, list)) and is_containing(other, (Variable, Node, types.GeneratorType)):  # non-unary table constraint
+            ll = flatten(other)
+            for i in range(len(ll)):  # we replace nodes by auxiliary variables if present
+                if isinstance(ll[i], Node):
+                    ll[i] = auxiliary().replace_node(ll[i])
+            queue_in.append((list(self), ll))
+            return True
         return self.__contains__(other)
 
     def _set_contains(self, other):  # for being able to use 'in' when expressing intension/extension constraints
@@ -97,7 +104,7 @@ def cursing():
         # if is_1d_tuple(other, Variable) or is_1d_list(other, Variable):  # non-unary table constraint
         #     queue_in.append((list(self), other))
         #     return True
-        if isinstance(other, (tuple, list)) and is_containing(other, (Variable, Node)):  # non-unary table constraint
+        if isinstance(other, (tuple, list)) and is_containing(other, (Variable, Node, types.GeneratorType)):  # non-unary table constraint
             ll = flatten(other)
             for i in range(len(ll)):  # we replace nodes by auxiliary variables if present
                 if isinstance(ll[i], Node):
@@ -371,8 +378,8 @@ class OpOverrider:
         return other.__ne__(self) if isinstance(other, (PartialConstraint, ScalarProduct)) else Node.build(TypeNode.NE, self, other)
 
     def __or__(self, other):
-        if isinstance(other, bool):
-            return self if other is False else True
+        # if isinstance(other, bool):
+        #     return self if other is False else True
         res = manage_global_indirection(self, other)
         if res is None:
             return functions.Or(self, other)
@@ -382,8 +389,8 @@ class OpOverrider:
         return Node.disjunction(self, other)
 
     def __and__(self, other):
-        if isinstance(other, bool):
-            return self if other is True else False
+        # if isinstance(other, bool):
+        #     return self if other is True else False
         res = manage_global_indirection(self, other)
         if res is None:
             return functions.And(self, other)
@@ -542,20 +549,19 @@ class ListInt(list):
                 # error("It seems that you build tables whose elements are lists instead of tuples: " + str(other) + " in " + str(self))
         return list.__contains__(self, other)
 
-
-def columns(m):
-    assert is_matrix(m), "columns() can only be called on matrices"
-    return ListVar(ListVar(row[j] for row in m) for j in range(len(m[0])))
+    def __str__(self):
+        return structured_list(self)
 
 
 class ListVar(list):
     # def __new__(self, variables):  # if we subclass tuple instead of list (while removing __init__)
     #     return super().__new__(ListVar, variables)
 
-    def __init__(self, variables):
+    def __init__(self, variables=[]):
         super().__init__(variables)
+        self.values = None
 
-    def __getslice__(self, i, j):
+    def __getslice__(self, i, j):  # TODO using getitem instead? as for ListCtr?
         return ListVar(super().__getslice__(i, j))
 
     def __add__(self, other):
@@ -579,9 +585,6 @@ class ListVar(list):
 
     # def __rmul__(self, other): return ListVar.__mul__(other, self)
 
-    def columns(self):
-        return columns(self)
-
     def around(self, i, j):
         assert is_matrix(self), "calling around should be made on a 2-dimensional array"
         n, m = len(self), len(self[i])
@@ -594,6 +597,21 @@ class ListVar(list):
         assert 0 <= i < n and 0 <= j < m
         return ListVar([self[i][j]] + [self[k][l] for k, l in [(i, j - 1), (i, j + 1), (i - 1, j), (i + 1, j)] if 0 <= k < n and 0 <= l < m])
 
+    def __str__(self):
+        return structured_list(self)
+
+
+class ListCtr(list):  # currently, mainly introduced for __str__ when calling posted()
+
+    def __init__(self, ectrs):
+        super().__init__(ectrs)
+
+    def __getitem__(self, k):
+        return ListCtr(super().__getitem__(k)) if isinstance(k, slice) else super().__getitem__(k)
+
+    def __str__(self):
+        return "\n".join(str(e) for e in self)
+
 
 def convert_to_namedtuples(obj):
     def with_only_alphanumeric_keys(obj):  # alphanum or '_'
@@ -647,6 +665,123 @@ def is_namedtuple(obj):  # imperfect way of checking, but must be enough for our
     fields = getattr(t, '_fields', None)
     return isinstance(fields, tuple) and all(type(field) == str for field in fields)
 
+
+def _list(t, mode):
+    return ListVar(v for v in t) if mode == 0 else ListInt(v for v in t) if mode == 1 else list(v for v in t)
+
+
+def columns(m):
+    """
+    Returns the transpose matrix of the specified matrix
+
+    :param m: a matrix (i.e. a two-dimensional list)
+    :return: the transpose matrix
+    """
+    assert is_matrix(m)
+    mode = 0 if is_matrix(m, Variable) else 1 if is_matrix(m, int) else 2
+    return _list((_list((row[j] for row in m), mode) for j in range(len(m[0]))), mode)
+
+
+def diagonal_down(m, i=-1, j=-1, check=True):
+    """
+    Returns the main downward diagonal, or another downward diagonal when the values
+    of the parameters i and j are not both equal to -1
+
+    :param m: a matrix (i.e. a two-dimensional list)
+    :param i: index of row (possibly -1)
+    :param j: index of column (possibly -1)
+    :param check: true when the structure of the matrix must be controlled
+    :return: the main downward diagonal (or another stipulated downward diagonal)
+    """
+    if check is True:
+        assert is_square_matrix(m), "The specified first parameter must be a square matrix."
+    if i == -1 and j == -1:
+        return diagonal_down(m, 0, 0, False)
+    mode = 0 if is_matrix(m, Variable) else 1 if is_matrix(m, int) else 2
+    if j == -1:
+        return _list((m[k][len(m) - (i - k) if k < i else k - i] for k in range(len(m))), mode)
+    return _list((m[i + k][j + k] for k in range(len(m) - max(i, j))), mode)
+
+
+def diagonals_down(m, *, broken=False):
+    """
+    Returns the list of downward diagonals of the specified matrix
+
+    :param m: a matrix (i.e. a two-dimensional list)
+    :param broken: true when broken diagonals must be completed
+    :return: the list of downward diagonals
+    """
+    assert is_square_matrix(m), "The specified first parameter must be a square matrix."
+    mode = 0 if is_matrix(m, Variable) else 1 if is_matrix(m, int) else 2
+    if broken:
+        return _list((diagonal_down(m, i, -1, False) for i in range(len(m))), mode)
+    return _list((diagonal_down(m, i, 0, False) for i in reversed(range(len(m) - 1))), mode) + \
+           _list((diagonal_down(m, 0, j, False) for j in range(1, len(m) - 1)), mode)
+
+
+def diagonal_up(m, i=-1, j=-1, check=True):
+    """
+       Returns the main upward diagonal, or another upward diagonal when the values
+       of the parameters i and j are not both equal to -1
+
+       :param m: a matrix (i.e. a two-dimensional list)
+       :param i: index of row (possibly -1)
+       :param j: index of column (possibly -1)
+       :param check: true when the structure of the matrix must be controlled
+       :return: the main upward diagonal (or another stipulated upward diagonal)
+       """
+    if check is True:
+        assert is_square_matrix(m), "The specified first parameter must be a square matrix."
+    if i == -1 and j == -1:
+        return diagonal_up(m, len(m) - 1, 0, False)
+    mode = 0 if is_matrix(m, Variable) else 1 if is_matrix(m, int) else 2
+    if j == -1:
+        return _list((m[k][len(m) - i - k - 1 if k < len(m) - i else 2 * len(m) - i - k - 1] for k in range(len(m))), mode)
+    return _list((m[i - k][j + k] for k in range(min(i + 1, len(m) - j))), mode)
+
+
+def diagonals_up(m, *, broken=False):
+    """
+      Returns the list of upward diagonals of the specified matrix
+
+      :param m: a matrix (i.e. a two-dimensional list)
+      :param broken: true when broken diagonals must be completed
+      :return: the list of upward diagonals
+      """
+    assert is_square_matrix(m), "The specified first parameter must be a square matrix."
+    mode = 0 if is_matrix(m, Variable) else 1 if is_matrix(m, int) else 2
+    if broken:
+        return _list((diagonal_up(m, i, -1, False) for i in range(len(m))), mode)
+    return _list((diagonal_up(m, i, 0, False) for i in range(1, len(m))), mode) + \
+           _list((diagonal_up(m, len(m) - 1, j, False) for j in range(1, len(m) - 1)), mode)
+
+
+def cp_array(*l):
+    """
+    Converts and returns a list containing integers into a list from the more specific type ListInt.
+    Converts and returns a list containing variables into a list from the more specific type ListVar.
+    Returns the same list in all other cases.
+    This method may be required for posting constraints Element.
+
+    :param l: a list (of any dimension)
+    :return: the same list, possibly converted into one of the two more specific types ListInt and ListVar
+    """
+    if len(l) == 1:
+        l = l[0]
+    if isinstance(l, (tuple, set, frozenset, types.GeneratorType)):
+        l = list(l)
+    assert isinstance(l, list) and len(l) > 0
+    if isinstance(l[0], (list, types.GeneratorType)):
+        assert all(isinstance(t, (list, types.GeneratorType)) for t in l)
+        res = [cp_array(t) for t in l]
+        return ListInt(res) if isinstance(res[0], ListInt) else ListVar(res)
+    if all(isinstance(v, int) for v in l):  # and None ?
+        return ListInt(l)
+    elif all(isinstance(v, Variable) for v in l):  # and None ?
+        return ListVar(l)
+    else:
+        raise NotImplemented
+
 # def to_special_list(t):
 #     assert is_1d_list(t)
 #     if is_containing(t, (Variable, type(None))):
